package org.squarebrackets;

import javax.annotation.Nonnull;

import static java.util.Objects.requireNonNull;

/**
 * Utility class for casting {@code Array}s.
 *
 * @author Leon van Zantvoort
 */
final class ArrayCast {

    /**
     * Suppresses default constructor, ensuring non-instantiability.
     */
    private ArrayCast() {
    }

    /**
     * Returns {@code BooleanArray} if {@code array} instanceof {@code BooleanArray}, {@code null} otherwise.
     *
     * @param array array to be casted.
     * @return casted array or null if array is not compatible.
     * @throws NullPointerException if specified array is null.
     */
    static BooleanArray toBooleanArray(@Nonnull Array<?> array) {
        if (array instanceof BooleanArray) {
            return (BooleanArray) array;
        }
        requireNonNull(array);
        return null;
    }

    /**
     * Returns {@code ByteArray} if {@code array} instanceof {@code ByteArray}, {@code null} otherwise.
     *
     * @param array array to be casted.
     * @return casted array or null if array is not compatible.
     * @throws NullPointerException if specified array is null.
     */
    static ByteArray toByteArray(@Nonnull Array<?> array) {
        if (array instanceof ByteArray) {
            return (ByteArray) array;
        }
        requireNonNull(array);
        return null;
    }

    /**
     * Returns {@code CharArray} if {@code array} instanceof {@code CharArray}, {@code null} otherwise.
     *
     * @param array array to be casted.
     * @return casted array or null if array is not compatible.
     * @throws NullPointerException if specified array is null.
     */
    static CharArray toCharArray(@Nonnull Array<?> array) {
        if (array instanceof CharArray) {
            return (CharArray) array;
        }
        requireNonNull(array);
        return null;
    }

    /**
     * Returns {@code ShortArray} if {@code array} instanceof {@code ShortArray}, {@code null} otherwise.
     *
     * @param array array to be casted.
     * @return casted array or null if array is not compatible.
     * @throws NullPointerException if specified array is null.
     */
    static ShortArray toShortArray(@Nonnull Array<?> array) {
        if (array instanceof ShortArray) {
            return (ShortArray) array;
        }
        requireNonNull(array);
        return null;
    }

    /**
     * Returns {@code IntArray} if {@code array} instanceof {@code IntArray}, {@code null} otherwise.
     *
     * @param array array to be casted.
     * @return casted array or null if array is not compatible.
     * @throws NullPointerException if specified array is null.
     */
    static IntArray toIntArray(@Nonnull Array<?> array) {
        if (array instanceof IntArray) {
            return (IntArray) array;
        }
        requireNonNull(array);
        return null;
    }

    /**
     * Returns {@code LongArray} if {@code array} instanceof {@code LongArray}, {@code null} otherwise.
     *
     * @param array array to be casted.
     * @return casted array or null if array is not compatible.
     * @throws NullPointerException if specified array is null.
     */
    static LongArray toLongArray(@Nonnull Array<?> array) {
        if (array instanceof LongArray) {
            return (LongArray) array;
        }
        requireNonNull(array);
        return null;
    }

    /**
     * Returns {@code FloatArray} if {@code array} instanceof {@code FloatArray}, {@code null} otherwise.
     *
     * @param array array to be casted.
     * @return casted array or null if array is not compatible.
     * @throws NullPointerException if specified array is null.
     */
    static FloatArray toFloatArray(@Nonnull Array<?> array) {
        if (array instanceof FloatArray) {
            return (FloatArray) array;
        }
        requireNonNull(array);
        return null;
    }

    /**
     * Returns {@code DoubleArray} if {@code array} instanceof {@code DoubleArray}, {@code null} otherwise.
     *
     * @param array array to be casted.
     * @return casted array or null if array is not compatible.
     * @throws NullPointerException if specified array is null.
     */
    static DoubleArray toDoubleArray(@Nonnull Array<?> array) {
        if (array instanceof DoubleArray) {
            return (DoubleArray) array;
        }
        requireNonNull(array);
        return null;
    }

    /**
     * Returns {@code ObjectArray} if {@code array} instanceof {@code ObjectArray}, {@code null} otherwise.
     *
     * @param array array to be casted.
     * @return casted array or null if array is not compatible.
     * @throws NullPointerException if specified array is null.
     */
    static ObjectArray<?> toObjectArray(@Nonnull Array<?> array) {
        if (array instanceof ObjectArray) {
            return (ObjectArray) array;
        }
        requireNonNull(array);
        return null;
    }
}
