/*
 * ============================================================================
 * GNU Lesser General Public License
 * ============================================================================
 *
 * Square Brackets - Java Array Framework.
 * Copyright (C) 2013  Beatgrid Media B.V.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 *
 * Beatgrid Media B.V.
 * Sisalbaan 5-A
 * 2352 AZ Leiderdorp
 * The Netherlands
 *
 * info@squarebrackets.org
 * http://squarebrackets.org
 */
package org.squarebrackets;

/**
 * <p>Base interface for capacity constrained mutable arrays.</p>
 *
 * <p>Bounded arrays have a maximum capacity. Elements can be added to, and removed from the tail of the array by
 * means of {@code add}, {@code remove} and {@code clear} methods. The the backing element store (typically an array or
 * buffer) reference is constant, defining the array's capacity constraints. The number of additional elements that the
 * array may store is returned by the {@code remainingCapacity} method. An array is bounded if it implements this
 * interface. In this case it must also return {@code true} for {@code hasCharacteristics(BOUNDED)}.
 *
 * <p>Bounded arrays are constructed by the static interface methods that are declared by the typed interfaces of the
 * array hierarchy. Each of these static methods provide different semantics, such as {@code valueOf} for wrapping
 * {@code Array} around native arrays and buffers and {@code copyOf} for copying the contents of native arrays or
 * framework {@code Array}s.
 *
 * <p>Arrays express characteristics to indicate that, for instance, elements are listed in sorted order, or no
 * duplicate elements exist. These characteristics allow array implementations to optimize their algorithms based
 * on the contents of the array. For example, operations that lookup elements can use a binary search algorithm if
 * the array is marked as being sorted. These optimizations may reduce the time complexity of such operations by
 * several orders of magnitude.
 * Bounded arrays always report {@code BOUNDED} and {@code MUTABLE}.</p>
 *
 * <p>The array implementations of the framework attempt to establish these characteristics to accommodate these
 * optimizations at best effort. This means that the array implementation determines which set of characteristics apply
 * to its elements without traversing these elements, in other words, without incurring a performance penalty.
 * Characteristics are always invalidated if conditions of these characteristics are broken as result of array
 * modification. Characteristics are only added in scenarios in which their conditions can be 'tested' without
 * traversal of elements. For example, the {@code SORTED} characteristic is added if the length of an array is reduced
 * to zero or one element, or if all elements are replaced by an array that is marked as being sorted.
 * The semantics of characteristics processing is documented for each "destructive" method. All implementations returned
 * by the framework adhere to these semantics. Custom implementations may choose to set characteristics at their
 * discretion as long as they do not violate the contract of these characteristics.
 * Users can take full control of characteristics assignment by means of the
 * {@link MutableArray#doAction(ArrayAction)} method.</p>
 *
 * @param <E> element type.
 * @author Leon van Zantvoort
 */
public interface BoundedArray<E> extends DynamicArray<E> {

}
